#include "Network.h"
#include "Helpers.h"
#include <fstream>
//#include <iostream>

using namespace std;

void Network::saveGraph(string filename)
{
	ofstream out((filename + ".dot").c_str());
	out << "digraph " << filename << endl << "{" << endl;
	for (int i = 0; i < layerCount; i++)
	{
		for (int j = 0; j < layers[i].neuronCount; j++)
		{
			//bias
			out << "\t" << (char)('a' + i) << j << " [label=\"";
			if (i != 0)
			       	out << layers[i].neurons[j].bias;
			out << "\"];" << endl;

			// weight
			if (i != layerCount - 1)
				for (int k = 0; k < layers[i + 1].neuronCount; k++)
					out << "\t" << (char)('a' + i) << j << " -> " << 
					(char)('a' + i + 1) << k << " [label=\"" << 
					layers[i].neurons[j].connections[k] << "\"];" << endl;
		}
	}
	out << "}" << endl;
}

void Network::save(string filename)
{
	ofstream out((filename + ".net").c_str());
	out << learningRate << endl;
	out << layerCount << endl;
	for (int i = 0; i < layerCount; i++)
		layers[i].save(out);
}

bool Network::load(string filename)
{
	ifstream in((filename + ".net").c_str());
//	if (!in.is_open())
	if (!in)
		return false;
	in >> learningRate;
	in >> layerCount;
	layers = new Layer[layerCount];
	for (int i = 0; i < layerCount; i++)
		layers[i].load(in);
	return true;
}

void Network::init(int layerCount, int *layerNeuronCount, double learningRate)
{
	if (layerCount < 2)
		return;

	this->layerCount = layerCount;
	layers = new Layer[layerCount];

	int i;
	for (i = 0; i < layerCount - 1; i++)
		layers[i].init(layerNeuronCount[i], layerNeuronCount[i + 1]);
	layers[i].init(layerNeuronCount[i], 0); // Last layer dont need connection

	this->learningRate = learningRate;
}

void Network::setInputLayer(double *in)
{
	for (int k = 0; k < inputLayer()->neuronCount; k++)
		inputLayer()->neurons[k].value = in[k];
}

double* Network::getOutputLayer()
{
	double *d = new double[outputLayer()->neuronCount];
	for (int k = 0; k < outputLayer()->neuronCount; k++)
		d[k] = outputLayer()->neurons[k].value;
	return d;
}

void Network::run()
{
	// We assumed that input layer is filled therefore
	// we begin from second layer
	for (int i = 1; i < layerCount; i++)
	{
		Layer *prev = layers + i - 1; // set prev to previous layer
		Layer *l = layers + i; // set l to current layer
		for (int j = 0; j < l->neuronCount; j++)
		{
			Neuron *n = l->neurons + j; // set n to current neuron
			n->value = n->bias;
			for (int k = 0; k < prev->neuronCount; k++)
				n->value += prev->neurons[k].value *
				prev->neurons[k].connections[j];

			normalize(n->value);
		}
	}
}

void Network::train(double *answer)
{
	for (int i = layerCount - 1; i > 0; i--) // foreach layers except first
	{
		for (int j = 0; j < layers[i].neuronCount; j++)
		{
			Neuron *n = layers[i].neurons + j;
			if (i == layerCount - 1)
			{
				n->delta = n->value * (1 - n->value) * (answer[j] - n->value);
				for (int k = 0; k < layers[i - 1].neuronCount; k++)
					layers[i - 1].neurons[k].connections[j] += n->delta * 
					learningRate * layers[i - 1].neurons[k].value;

				n->bias += n->delta * learningRate;
			}
			else
			{
				double s = 0;
				for (int k = 0; k < layers[i + 1].neuronCount; k++)
					s += n->connections[k] * layers[i + 1].neurons[k].delta;
				n->delta = n->value * (1 - n->value) * s;

				for (int k = 0; k < layers[i - 1].neuronCount; k++)
					layers[i - 1].neurons[k].connections[j] += n->delta * 
					learningRate * layers[i - 1].neurons[k].value;

				n->bias += n->delta * learningRate;
			}
		}
	}
}

