#include <iostream>
#include <string>
#include <ctime>
#include <cmath>
#include <fstream>
#include "Network.h"

using namespace std;

const int netin = 8;
const int netout = 4;
const string prgName = "sumAuto";

void trainNet(Network &net, int exampleCount, double inp[][netin], double outp[][netout], double accuracy = 1e-2)
{
	double sError, err;

	time_t start = time(NULL);

	accuracy *= accuracy;

	int a;
	for (a = 1, sError = 999; sError > accuracy; a++)
	{
		if (a % 50000 == 0)
		{
			cout.width(2);
			cout << a / 50000;
			cout << "    " << sError << endl;
		}

		sError = 0;

		for (int i = 0; i < exampleCount; i++)
		{
			net.setInputLayer(inp[i]);
			net.run();

			for (int j = 0; j < net.outputLayer()->neuronCount; j++)
			{
				err = outp[i][j] - net.outputLayer()->neurons[j].value; 
				sError += err * err;
			}

			net.train(outp[i]);
		}
	}

	cout << "Trained " << a - 1 << " times" << endl << "Finished in " << difftime(time(NULL), start) << " second(s)" << endl;
}


void showAnswer(Network &net, int exampleCount, double inp[][netin], double outp[][netout], bool showError = true)
{
	double sError = 0, err;

	for (int i = 0; i < exampleCount; i++)
	{
		cout << endl;

		cout << "Input:";
		for (int j = 0; j < net.inputLayer()->neuronCount; j++)
			cout << " " << inp[i][j];
		cout << endl;

		net.setInputLayer(inp[i]);
		net.run();

		cout << "Output:";
		err = 0;
		for (int j = 0; j < net.outputLayer()->neuronCount; j++)
		{
			if (showError)
				err += fabs(outp[i][j] - net.outputLayer()->neurons[j].value); 
			cout << " " << net.outputLayer()->neurons[j].value;
		}
		cout << endl;

		if (showError)
		{
			cout << "Error: " << err << endl;
			sError += err;
		}
	}

	if (showError)
		cout << endl << "Average error: " << sError / exampleCount << endl;

}

int main(int argc)
{
//	srand(1234);
	srand(time(NULL));


	Network net;

	double acc = 0;
	cout << "Enter accuracy: ";
	cin >> acc;

	if (!net.load(prgName)) // if there isn't a net create a random initialized network
	{
		int u;
		cout << "Enter hidden layer neurons: ";
		cin >> u;
		int layers[] = {netin, u, netout};
		net.init(3, layers, 1);
	}

	for(;;)
	{

	double inp[1000][netin];
	double outp[1000][netout];

	ifstream fin((prgName + ".in").c_str());

	int l = 0; // Samples count
	int x, y, z;
	while (fin >> x >> y >> z)
	{
		for (int i = netin - 1; i >= netin / 2; i--)
		{
			inp[l][i] = x % 2;
			x /= 2;
		}

		for (int i = netin / 2 - 1; i >= 0; i--)
		{
			inp[l][i] = y % 2;
			y /= 2;
		}

		for (int i = netout - 1; i >= 0; i--)
		{
			outp[l][i] = z % 2;
			z /= 2;
		}
		l++;
	}
	fin.close();

	trainNet(net, l, inp, outp, acc);
	net.save(prgName);
	net.saveGraph(prgName);

	do
	{
		x = rand() % 16;
		y = rand() % 16;
	} while (x + y >= 16);
	cout << "Using two number: " << x << " " << y << endl;
	int xx, yy;
	xx = x;
	yy = y;

	for (int i = netin - 1; i >= netin / 2; i--)
	{
		net.inputLayer()->neurons[i].value = x % 2;
		x /= 2;
	}

	for (int i = netin / 2 - 1; i >= 0; i--)
	{
		net.inputLayer()->neurons[i].value = y % 2;
		y /= 2;
	}

	net.run();

	int sum = 0;
	for (int i = netout - 1, p = 1; i >= 0; i--, p *= 2)
		sum += (int)round(net.outputLayer()->neurons[i].value) * p;
	cout << "Answer: " << sum << endl;

	if (sum != xx + yy)
	{
		ofstream fout((prgName + ".in").c_str(), ios::ate | ios::app);
		fout << xx << " " << yy << " " << xx + yy << endl;
		cout << "Ooops wrong answer... Added to train file" << endl;
		fout.close();
	}
	else
		cout << "Correct ;)" << endl;

	}

	return 0;
}

