#include <iostream>
#include <cstdlib>
#include <cmath>
#include <fstream>
#include <string>
#include "Network.h"

using namespace std;

const int netin = 2;
const int netout = 1;
const string prgName = "xor";

void trainNet(Network &net, int exampleCount, double inp[][netin], double outp[][netout], double accuracy = 1e-2)
{
	double sError, err;

	time_t start = time(NULL);

	accuracy *= accuracy;

	int a;
	for (a = 1, sError = 1; sError > accuracy; a++)
	{
		if (a % 100000 == 0)
		{
			cout.width(2);
			cout << a / 100000;
			cout << "    " << sError << endl;
		}

		sError = 0;

		for (int i = 0; i < exampleCount; i++)
		{
			net.setInputLayer(inp[i]);
			net.run();

			for (int j = 0; j < net.outputLayer()->neuronCount; j++)
			{
				err = outp[i][j] - net.outputLayer()->neurons[j].value; 
				sError += err * err;
			}

			net.train(outp[i]);
		}
	}

	cout << "Trained " << a - 1 << " times" << endl << "Finished in " << difftime(time(NULL), start) << " second(s)" << endl;
}

void showAnswer(Network &net, int exampleCount, double inp[][netin], double outp[][netout], bool showError = true)
{
	double sError = 0, err;

	for (int i = 0; i < exampleCount; i++)
	{
		cout << endl;

		cout << "Input:";
		for (int j = 0; j < net.inputLayer()->neuronCount; j++)
			cout << " " << inp[i][j];
		cout << endl;

		net.setInputLayer(inp[i]);
		net.run();

		cout << "Output:";
		err = 0;
		for (int j = 0; j < net.outputLayer()->neuronCount; j++)
		{
			if (showError)
				err += fabs(outp[i][j] - net.outputLayer()->neurons[j].value); 
			cout << " " << net.outputLayer()->neurons[j].value;
		}
		cout << endl;

		if (showError)
		{
			cout << "Error: " << err << endl;
			sError += err;
		}
	}

	if (showError)
		cout << endl << "Average error: " << sError / exampleCount << endl;

}

int main(int argc, char **argv)
{
	srand(1234);
//	srand(time(NULL));

	Network net;
	int layers[] = {2, 2, 1};

	double inp[][2] = {{0, 0}, {1, 0}, {0, 1}, {1, 1}};
	double outp[][1] = {{0}, {1}, {1}, {0}};

	if (argc > 1)
	{
		net.init(3, layers, 1);
		trainNet(net, 4, inp, outp, 1e-2);
		net.save("xor");
	}
	else
		if (!net.load(prgName))
		{
			cout << "Can't open `" << prgName << ".net' try `" << argv[0] << " train'" << endl;
			return 1;
		}

	net.saveGraph(prgName);
	showAnswer(net, 4, inp, outp);

	return 0;
}

